/*
Section:  1) 0NF to 1NF
Script:   4) Getting Data Into 1NF -- Final
Basics:   Our first attempt at data loading was easy but
			led to processing nightmares.  We are now going
			to turn this structure into a first normal form model.
Notes:    
*/

/*
This time around, we're going to model entities and attributes.
This means our final product might NOT look like the spreadsheet.
We want to keep at least the data points we have, but could also
include other points if we need them.
For this 1NF model, we'll stick to using natural keys.

LineupMember
	- Year (PK)
	- Name (PK)
	- IsSessionOnly
	- (Set of instruments)

Instrument
	- Name (PK)

Song
	- Year
	- Type { Studio, Live }
	- AlbumName (PK, AK2)
	- Label
	- TrackNumber (AK2)	<-- New attribute!
	- SongName (PK)
	- (Set of members)

ProjeKctMember
	- Year (PK)
	- Name (PK)
	- (Set of instruments)

ProjeKctTrack
	- Year
	- Type { Studio, Live }
	- ProjeKctName
	- AlbumName (PK, AK2)
	- TrackNumber (AK2)	<-- New attribute!
	- TrackName (PK)
	- (Set of projeKct members)

Each time we see "(Set of...)" we have uncovered a
many-to-many relationship and need to create a junction table.

TO DO:
1)  Figure out candidate keys for each entity.
2)  Use those candidate keys to create the junction tables
*/

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'LineupMember'
		AND OBJECT_SCHEMA_NAME(object_id) = '1NF'
)
BEGIN
	CREATE TABLE [1NF].[LineupMember]
	(
		[Year] INT NOT NULL,
		[Name] VARCHAR(40) NOT NULL,
		[IsSessionOnly] BIT NOT NULL
	);

	ALTER TABLE [1NF].[LineupMember] ADD CONSTRAINT [PK_1NF_LineupMember] PRIMARY KEY CLUSTERED(Year, Name);
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'Instrument'
		AND OBJECT_SCHEMA_NAME(object_id) = '1NF'
)
BEGIN
	CREATE TABLE [1NF].[Instrument]
	(
		[Name] VARCHAR(40) NOT NULL
	);

	ALTER TABLE [1NF].[Instrument] ADD CONSTRAINT [PK_1NF_Instrument] PRIMARY KEY CLUSTERED(Name);
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'LineupMemberXInstrument'
		AND OBJECT_SCHEMA_NAME(object_id) = '1NF'
)
BEGIN
	CREATE TABLE [1NF].[LineupMemberXInstrument]
	(
		[LineupMemberYear] INT NOT NULL,
		[LineupMemberName] VARCHAR(40) NOT NULL,
		[InstrumentName] VARCHAR(40) NOT NULL
	);

	ALTER TABLE [1NF].[LineupMemberXInstrument] ADD CONSTRAINT [PK_1NF_LineupMemberXInstrument] PRIMARY KEY CLUSTERED(LineupMemberYear, LineupMemberName, InstrumentName);
	ALTER TABLE [1NF].[LineupMemberXInstrument] ADD CONSTRAINT [FK_LineupMemberXInstrument_LineupMember] FOREIGN KEY(LineupMemberYear, LineupMemberName) REFERENCES [1NF].[LineupMember](Year, Name) ON UPDATE CASCADE;
	ALTER TABLE [1NF].[LineupMemberXInstrument] ADD CONSTRAINT [FK_LineupMemberXInstrument_Instrument] FOREIGN KEY(InstrumentName) REFERENCES [1NF].[Instrument](Name) ON UPDATE CASCADE;
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'Song'
		AND OBJECT_SCHEMA_NAME(object_id) = '1NF'
)
BEGIN
	CREATE TABLE [1NF].[Song]
	(
		[Year] INT NOT NULL,
		[Type] VARCHAR(6) NOT NULL,
		[AlbumName] VARCHAR(120) NOT NULL,
		[Label] VARCHAR(50) NOT NULL,
		[TrackNumber] TINYINT NOT NULL,
		[SongName] VARCHAR(120) NOT NULL
	);

	ALTER TABLE [1NF].[Song] ADD CONSTRAINT [PK_1NF_Song] PRIMARY KEY CLUSTERED(AlbumName, SongName);
	ALTER TABLE [1NF].[Song] ADD CONSTRAINT [UKC_1NF_Song] UNIQUE(AlbumName, TrackNumber);
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'LineupMemberXSong'
		AND OBJECT_SCHEMA_NAME(object_id) = '1NF'
)
BEGIN
	CREATE TABLE [1NF].[LineupMemberXSong]
	(
		[LineupMemberYear] INT NOT NULL,
		[LineupMemberName] VARCHAR(40) NOT NULL,
		[AlbumName] VARCHAR(120) NOT NULL,
		[SongName] VARCHAR(120) NOT NULL
	);

	ALTER TABLE [1NF].[LineupMemberXSong] ADD CONSTRAINT [PK_1NF_LineupMemberXSong] PRIMARY KEY CLUSTERED(LineupMemberYear, LineupMemberName, AlbumName, SongName);
	ALTER TABLE [1NF].[LineupMemberXSong] ADD CONSTRAINT [FK_LineupMemberXSong_LineupMember] FOREIGN KEY(LineupMemberYear, LineupMemberName) REFERENCES [1NF].[LineupMember](Year, Name) ON UPDATE CASCADE;
	ALTER TABLE [1NF].[LineupMemberXSong] ADD CONSTRAINT [FK_LineupMemberXSong_Song] FOREIGN KEY(AlbumName, SongName) REFERENCES [1NF].[Song](AlbumName, SongName) ON UPDATE CASCADE;
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'ProjeKctMember'
		AND OBJECT_SCHEMA_NAME(object_id) = '1NF'
)
BEGIN
	CREATE TABLE [1NF].[ProjeKctMember]
	(
		[Year] INT NOT NULL,
		[Name] VARCHAR(40) NOT NULL
	);

	ALTER TABLE [1NF].[ProjeKctMember] ADD CONSTRAINT [PK_1NF_ProjeKctMember] PRIMARY KEY CLUSTERED(Year, Name);
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'ProjeKctMemberXInstrument'
		AND OBJECT_SCHEMA_NAME(object_id) = '1NF'
)
BEGIN
	CREATE TABLE [1NF].[ProjeKctMemberXInstrument]
	(
		[ProjeKctMemberYear] INT NOT NULL,
		[ProjeKctMemberName] VARCHAR(40) NOT NULL,
		[InstrumentName] VARCHAR(40) NOT NULL
	);

	ALTER TABLE [1NF].[ProjeKctMemberXInstrument] ADD CONSTRAINT [PK_1NF_ProjeKctMemberXInstrument] PRIMARY KEY CLUSTERED(ProjeKctMemberYear, ProjeKctMemberName, InstrumentName);
	ALTER TABLE [1NF].[ProjeKctMemberXInstrument] ADD CONSTRAINT [FK_ProjeKctMemberXInstrument_ProjeKctMember] FOREIGN KEY(ProjeKctMemberYear, ProjeKctMemberName) REFERENCES [1NF].[ProjeKctMember](Year, Name) ON UPDATE CASCADE;
	ALTER TABLE [1NF].[ProjeKctMemberXInstrument] ADD CONSTRAINT [FK_ProjeKctMemberXInstrument_Instrument] FOREIGN KEY(InstrumentName) REFERENCES [1NF].[Instrument](Name) ON UPDATE CASCADE;
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'ProjeKctTrack'
		AND OBJECT_SCHEMA_NAME(object_id) = '1NF'
)
BEGIN
	CREATE TABLE [1NF].[ProjeKctTrack]
	(
		[Year] INT NOT NULL,
		[Type] VARCHAR(6) NOT NULL,
		[ProjeKctName] VARCHAR(120) NOT NULL,
		[AlbumName] VARCHAR(120) NOT NULL,
		[TrackNumber] TINYINT NOT NULL,
		[TrackName] VARCHAR(120) NOT NULL
	);

	ALTER TABLE [1NF].[ProjeKctTrack] ADD CONSTRAINT [PK_1NF_ProjeKctTrack] PRIMARY KEY CLUSTERED(AlbumName, TrackName);
	ALTER TABLE [1NF].[ProjeKctTrack] ADD CONSTRAINT [UKC_1NF_ProjeKctTrack] UNIQUE(AlbumName, TrackNumber);
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'ProjeKctMemberXProjeKctTrack'
		AND OBJECT_SCHEMA_NAME(object_id) = '1NF'
)
BEGIN
	CREATE TABLE [1NF].[ProjeKctMemberXProjeKctTrack]
	(
		[ProjeKctMemberYear] INT NOT NULL,
		[ProjeKctMemberName] VARCHAR(40) NOT NULL,
		[AlbumName] VARCHAR(120) NOT NULL,
		[TrackName] VARCHAR(120) NOT NULL
	);

	ALTER TABLE [1NF].[ProjeKctMemberXProjeKctTrack] ADD CONSTRAINT [PK_1NF_ProjeKctMemberXProjeKctTrack] PRIMARY KEY CLUSTERED(ProjeKctMemberYear, ProjeKctMemberName, AlbumName, TrackName);
	ALTER TABLE [1NF].[ProjeKctMemberXProjeKctTrack] ADD CONSTRAINT [FK_ProjeKctMemberXProjeKctTrack_ProjeKctMember] FOREIGN KEY(ProjeKctMemberYear, ProjeKctMemberName) REFERENCES [1NF].[ProjeKctMember](Year, Name) ON UPDATE CASCADE;
	ALTER TABLE [1NF].[ProjeKctMemberXProjeKctTrack] ADD CONSTRAINT [FK_ProjeKctMemberXProjeKctTrack_ProjeKctTrack] FOREIGN KEY(AlbumName, TrackName) REFERENCES [1NF].[ProjeKctTrack](AlbumName, TrackName) ON UPDATE CASCADE;
END
GO


--Now let's insert some data and see how answering our previous questions becomes easier.
MERGE [1NF].[Instrument] AS tgt
USING
(
	SELECT
		Name
	FROM
	(
		VALUES
		('Bass'),
		('Stick'),
		('Synth'),
		('Touch guitar'),
		('Drums'),
		('Guitar'),
		('V-Drums'),
		('Talker'),
		('Guitar synth')
	) v(Name)
) AS src
	ON tgt.Name = src.Name
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	Name
)
VALUES
(
	src.Name
);

MERGE [1NF].[ProjeKctMember] AS tgt
USING
(
	SELECT
		Year,
		Name
	FROM
	(
		VALUES
		(1998, 'Tony Levin'),
		(1998, 'Trey Gunn'),
		(1998, 'Bill Bruford'),
		(1998, 'Robert Fripp'),
		(1997, 'Adrian Belew'),
		(1997, 'Robert Fripp'),
		(1997, 'Trey Gunn')
	) v(Year, Name)
) AS src
	ON tgt.Name = src.Name
	AND tgt.Year = src.Year
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	Year,
	Name
)
VALUES
(
	src.Year,
	src.Name
);

MERGE [1NF].[ProjeKctMemberXInstrument] AS tgt
USING
(
	SELECT
		Year,
		Name,
		InstrumentName
	FROM
	(
		VALUES
		(1998, 'Tony Levin', 'Bass'),
		(1998, 'Tony Levin', 'Stick'),
		(1998, 'Tony Levin', 'Synth'),
		(1998, 'Trey Gunn', 'Touch guitar'),
		(1998, 'Bill Bruford', 'Drums'),
		(1998, 'Robert Fripp', 'Guitar'),
		(1997, 'Adrian Belew', 'V-Drums'),
		(1997, 'Robert Fripp', 'Guitar'),
		(1997, 'Trey Gunn', 'Touch guitar'),
		(1997, 'Trey Gunn', 'Talker'),
		(1997, 'Trey Gunn', 'Guitar synth')
	) v(Year, Name, InstrumentName)
) AS src
	ON tgt.ProjeKctMemberYear = src.Year
	AND tgt.ProjeKctMemberName = src.Name
	AND tgt.InstrumentName = src.InstrumentName
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	ProjeKctMemberYear,
	ProjeKctMemberName,
	InstrumentName
)
VALUES
(
	src.Year,
	src.Name,
	src.InstrumentName
);

MERGE [1NF].[ProjeKctTrack] AS tgt
USING
(
	SELECT
		[Year],
		[Type],
		[ProjeKctName],
		[AlbumName],
		[TrackNumber],
		[TrackName]
	FROM
	(
		VALUES
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', 1, '4 i 1'),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', 2, '4 ii 1'),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', 3, '1 ii 2'),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', 4, '4 ii 4'),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', 5, '2 ii 3'),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', 6, '3 i 2'),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', 7, '3 ii 2'),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', 8, '2 ii 4'),
		(1998, 'Live', 'ProjeKct 1', 'Live at the Jazz Cafe', 9, '4 i 3'),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 1, 'Sus-tayn-Z'),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 2, 'Heavy ConstruKction'),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 3, 'The Deception of the Thrush'),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 4, 'X-chayn-jiZ'),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 5, 'Light ConstruKction'),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 6, 'Vector Shift to Planet Detroit'),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 7, 'Contrary ConstruKction'),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 8, 'Live Groove'),
		(1997, 'Live', 'ProjeKct 2', 'Live Groove', 9, 'Vector Shift to Planet Belewbeloid'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 1, 'Space Groove II'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 2, 'Space Groove III'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 3, 'Space Groove I'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 4, 'The Planet Zarg Quartet'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 5, 'Happy Hour on Planet Zarg'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 6, 'Is There Life on Planet Zarg?'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 7, 'Low Life in Sector Q-3'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 8, 'Sector Shift'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 9, 'Laura in Space'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 10, 'Sector Drift'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 11, 'Sector Patrol'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 12, 'In Space There Is No North...'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 13, 'Vector Patrol'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 14, 'Deserts of Arcadia (North)'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 15, 'Deserts of Arcadia (South)'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 16, 'Snake Drummers of Sector Q-e'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 17, 'Escape from Sagittarius A'),
		(1997, 'Studio', 'ProjeKct 2', 'Space Groove', 18, 'Return to Station B')
	) v
	(
		[Year],
		[Type],
		[ProjeKctName],
		[AlbumName],
		[TrackNumber],
		[TrackName]
	)
) AS src
	ON tgt.[AlbumName] = src.[AlbumName]
	AND tgt.[TrackName] = src.[TrackName]
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	[Year],
	[Type],
	[ProjeKctName],
	[AlbumName],
	[TrackNumber],
	[TrackName]
)
VALUES
(
	src.Year,
	src.Type,
	src.ProjeKctName,
	src.AlbumName,
	src.TrackNumber,
	src.TrackName
);

MERGE [1NF].[ProjeKctMemberXProjeKctTrack] AS tgt
USING
(
	SELECT
		pm.Year AS ProjeKctMemberYear,
		pm.Name AS ProjeKctMemberName,
		pt.AlbumName,
		pt.TrackName
	FROM
		[1NF].[ProjeKctMember] pm
		INNER JOIN [1NF].[ProjeKctTrack] pt
			ON pm.Year = pt.Year
) AS src
	ON tgt.ProjeKctMemberYear = src.ProjeKctMemberYear
	AND tgt.ProjeKctMemberName = src.ProjeKctMemberName
	AND tgt.AlbumName = src.AlbumName
	AND tgt.TrackName = src.TrackName
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	[ProjeKctMemberYear],
	[ProjeKctMemberName],
	[AlbumName],
	[TrackName]
)
VALUES
(
	src.[ProjeKctMemberYear],
	src.[ProjeKctMemberName],
	src.[AlbumName],
	src.[TrackName]
);



--1)  Provide a list of instruments Tony Levin played on ProjeKct 1 as a set.
--		Lots of joins, but the query is much simpler and this is the type of
--		query we can optimize and maintain easily.
--		Note the DISTINCT in there.  This could be a code smell, but we're not 
--		able to make that decision just yet.
SELECT DISTINCT
	i.Name
FROM [1NF].[ProjeKctMemberXInstrument] pxi
	INNER JOIN [1NF].[ProjeKctMember] pm
		ON pxi.ProjeKctMemberYear = pm.Year
		AND pxi.ProjeKctMemberName = pm.Name
	INNER JOIN [1NF].[Instrument] i
		ON pxi.InstrumentName = i.Name
	INNER JOIN [1NF].[ProjeKctMemberXProjeKctTrack] pxp
		ON pm.Year = pxp.ProjeKctMemberYear
		AND pm.Name = pxp.ProjeKctMemberName
	INNER JOIN [1NF].[ProjeKctTrack] pt
		ON pxp.TrackName = pt.TrackName
		AND pxp.AlbumName = pt.AlbumName
WHERE
	pm.Name = 'Tony Levin'
	AND pt.ProjeKctName = 'ProjeKct 1';

--2) Who played on ProjeKct 2?
SELECT DISTINCT
	pm.Name
FROM [1NF].[ProjeKctMember] pm
	INNER JOIN [1NF].[ProjeKctMemberXProjeKctTrack] pxp
		ON pm.Year = pxp.ProjeKctMemberYear
		AND pm.Name = pxp.ProjeKctMemberName
	INNER JOIN [1NF].[ProjeKctTrack] pt
		ON pxp.TrackName = pt.TrackName
		AND pxp.AlbumName = pt.AlbumName
WHERE
	pt.ProjeKctName = 'ProjeKct 2';

--3)  Which instruments did Trey Gunn play on the Space Groove album?
--		Uh-oh:  our data model isn't right:  it attributes 3 instruments
--		to Gunn when he really only played 2.  Let's shelve that for the moment...
SELECT DISTINCT
	i.Name
FROM [1NF].[ProjeKctMemberXInstrument] pxi
	INNER JOIN [1NF].[ProjeKctMember] pm
		ON pxi.ProjeKctMemberYear = pm.Year
		AND pxi.ProjeKctMemberName = pm.Name
	INNER JOIN [1NF].[Instrument] i
		ON pxi.InstrumentName = i.Name
	INNER JOIN [1NF].[ProjeKctMemberXProjeKctTrack] pxp
		ON pm.Year = pxp.ProjeKctMemberYear
		AND pm.Name = pxp.ProjeKctMemberName
	INNER JOIN [1NF].[ProjeKctTrack] pt
		ON pxp.TrackName = pt.TrackName
		AND pxp.AlbumName = pt.AlbumName
WHERE
	pm.Name = 'Trey Gunn'
	AND pt.AlbumName = 'Space Groove';