/*
Section:  3) 2NF to 3NF
Script:   8) Getting Data Into 3NF -- Final
Basics:   We have our data set in 2NF.  Now to move to the next step.
Notes:    
*/

/*
Once we're in 2NF, going to 3NF means moving non-key attributes
which are facts about other non-key attributes off into separate
tables.

NEW structure:

Member
	- Name (PK)

Instrument
	- Name (PK)

- Label
	- Name (PK)
	- Headquarters

Album
	- Year
	- ProjeKctName (NULL)
	- Type { Studio, Live }
	- AlbumName (PK)
	- LabelName (NULL)

AlbumXMember
	- AlbumName (PK)
	- MemberName (PK)
	- IsSessionOnly

AlbumXMemberXInstrument
	- AlbumName (PK)
	- MemberName (PK)
	- InstrumentName (PK)

Song
	- AlbumName (PK, AK2)
	- TrackNumber (AK2)
	- SongName (PK)

SongXMember
	- AlbumName (PK)
	- SongName (PK)
	- MemberName (PK)
*/

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'Member'
		AND OBJECT_SCHEMA_NAME(object_id) = '3NF'
)
BEGIN
	CREATE TABLE [3NF].[Member]
	(
		[Name] VARCHAR(40) NOT NULL
	);

	ALTER TABLE [3NF].[Member] ADD CONSTRAINT [PK_3NF_Member] PRIMARY KEY CLUSTERED(Name);
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'Label'
		AND OBJECT_SCHEMA_NAME(object_id) = '3NF'
)
BEGIN
	CREATE TABLE [3NF].[Label]
	(
		[Name] VARCHAR(50) NOT NULL,
		[Headquarters] VARCHAR(50) NOT NULL
	);

	ALTER TABLE [3NF].[Label] ADD CONSTRAINT [PK_3NF_Label] PRIMARY KEY CLUSTERED(Name);
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'Album'
		AND OBJECT_SCHEMA_NAME(object_id) = '3NF'
)
BEGIN
	CREATE TABLE [3NF].[Album]
	(
		[Name] VARCHAR(120) NOT NULL,
		[ProjeKctName] VARCHAR(50) NULL,
		[Year] INT NOT NULL,
		[Type] VARCHAR(6) NOT NULL,
		[LabelName] VARCHAR(50) NULL
	);

	ALTER TABLE [3NF].[Album] ADD CONSTRAINT [PK_3NF_Album] PRIMARY KEY CLUSTERED(Name);
	ALTER TABLE [3NF].[Album] ADD CONSTRAINT [FK_Album_Label] FOREIGN KEY(LabelName) REFERENCES [3NF].[Label](Name) ON UPDATE CASCADE;
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'AlbumXMember'
		AND OBJECT_SCHEMA_NAME(object_id) = '3NF'
)
BEGIN
	CREATE TABLE [3NF].[AlbumXMember]
	(
		[AlbumName] VARCHAR(120) NOT NULL,
		[MemberName] VARCHAR(40) NOT NULL,
		[IsSessionOnly] BIT NOT NULL
	);

	ALTER TABLE [3NF].[AlbumXMember] ADD CONSTRAINT [PK_3NF_AlbumXMember] PRIMARY KEY CLUSTERED(AlbumName, MemberName);
	ALTER TABLE [3NF].[AlbumXMember] ADD CONSTRAINT [FK_AlbumXMember_Album] FOREIGN KEY(AlbumName) REFERENCES [3NF].[Album](Name) ON UPDATE CASCADE;
	ALTER TABLE [3NF].[AlbumXMember] ADD CONSTRAINT [FK_AlbumXMember_Member] FOREIGN KEY(MemberName) REFERENCES [3NF].[Member](Name) ON UPDATE CASCADE;
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'Instrument'
		AND OBJECT_SCHEMA_NAME(object_id) = '3NF'
)
BEGIN
	CREATE TABLE [3NF].[Instrument]
	(
		[Name] VARCHAR(40) NOT NULL
	);

	ALTER TABLE [3NF].[Instrument] ADD CONSTRAINT [PK_3NF_Instrument] PRIMARY KEY CLUSTERED(Name);
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'AlbumXMemberXInstrument'
		AND OBJECT_SCHEMA_NAME(object_id) = '3NF'
)
BEGIN
	CREATE TABLE [3NF].[AlbumXMemberXInstrument]
	(
		[AlbumName] VARCHAR(120) NOT NULL,
		[MemberName] VARCHAR(40) NOT NULL,
		[InstrumentName] VARCHAR(40) NOT NULL
	);

	ALTER TABLE [3NF].[AlbumXMemberXInstrument] ADD CONSTRAINT [PK_3NF_AlbumXMemberXInstrument] PRIMARY KEY CLUSTERED(AlbumName, MemberName, InstrumentName);
	ALTER TABLE [3NF].[AlbumXMemberXInstrument] ADD CONSTRAINT [FK_AlbumXMemberXInstrument_Album] FOREIGN KEY(AlbumName) REFERENCES [3NF].[Album](Name) ON UPDATE CASCADE;
	ALTER TABLE [3NF].[AlbumXMemberXInstrument] ADD CONSTRAINT [FK_AlbumXMemberXInstrument_Member] FOREIGN KEY(MemberName) REFERENCES [3NF].[Member](Name) ON UPDATE CASCADE;
	ALTER TABLE [3NF].[AlbumXMemberXInstrument] ADD CONSTRAINT [FK_AlbumXMemberXInstrument_Instrument] FOREIGN KEY(InstrumentName) REFERENCES [3NF].[Instrument](Name) ON UPDATE CASCADE;
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'Song'
		AND OBJECT_SCHEMA_NAME(object_id) = '3NF'
)
BEGIN
	CREATE TABLE [3NF].[Song]
	(
		[AlbumName] VARCHAR(120) NOT NULL,
		[TrackNumber] TINYINT NOT NULL,
		[SongName] VARCHAR(120) NOT NULL
	);

	ALTER TABLE [3NF].[Song] ADD CONSTRAINT [PK_3NF_Song] PRIMARY KEY CLUSTERED(AlbumName, SongName);
	ALTER TABLE [3NF].[Song] ADD CONSTRAINT [UKC_3NF_Song] UNIQUE(AlbumName, TrackNumber);
	ALTER TABLE [3NF].[Song] ADD CONSTRAINT [FK_Song_Album] FOREIGN KEY(AlbumName) REFERENCES [3NF].[Album](Name) ON UPDATE CASCADE;
END
GO

IF NOT EXISTS
(
	SELECT 1
	FROM sys.tables
	WHERE
		name = 'SongXMember'
		AND OBJECT_SCHEMA_NAME(object_id) = '3NF'
)
BEGIN
	CREATE TABLE [3NF].[SongXMember]
	(
		[AlbumName] VARCHAR(120) NOT NULL,
		[SongName] VARCHAR(120) NOT NULL,
		[MemberName] VARCHAR(40) NOT NULL,
	);

	ALTER TABLE [3NF].[SongXMember] ADD CONSTRAINT [PK_3NF_SongXMember] PRIMARY KEY CLUSTERED(AlbumName, SongName, MemberName);
	ALTER TABLE [3NF].[SongXMember] ADD CONSTRAINT [FK_SongXMember_Member] FOREIGN KEY(MemberName) REFERENCES [3NF].[Member](Name) ON UPDATE CASCADE;
	ALTER TABLE [3NF].[SongXMember] ADD CONSTRAINT [FK_SongXMember_Song] FOREIGN KEY(AlbumName, SongName) REFERENCES [3NF].[Song](AlbumName, SongName) ON UPDATE CASCADE;
END
GO

--Now let's insert some data and answer the previous questions once more.
MERGE [3NF].[Instrument] AS tgt
USING
(
	SELECT
		Name
	FROM
	(
		VALUES
		('Bass'),
		('Devices'),
		('Drums'),
		('Flute'),
		('Guitar'),
		('Guitar synth'),
		('Keyboards'),
		('Mellotron'),
		('Percussion'),
		('Piano'),
		('Reeds'),
		('Sax'),
		('Stick'),
		('Synth'),
		('Talker'),
		('Touch guitar'),
		('Vocals'),
		('V-Drums'),
		('Vibes'),
		('Woodwind'),
		('Words'),
		('Words and Illumination')
		
	) v(Name)
) AS src
	ON tgt.Name = src.Name
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	Name
)
VALUES
(
	src.Name
);

MERGE [3NF].[Member] AS tgt
USING
(
	SELECT
		Name
	FROM
	(
		VALUES
		('Tony Levin'),
		('Trey Gunn'),
		('Bill Bruford'),
		('Robert Fripp'),
		('Adrian Belew'),
		('Ian McDonald'),
		('Greg Lake'),
		('Michael Giles'),
		('Peter Sinfield'),
		('Peter Giles'),
		('Keith Tippett'),
		('Mel Collins'),
		('Gordon Haskell')
	) v(Name)
) AS src
	ON tgt.Name = src.Name
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	Name
)
VALUES
(
	src.Name
);

MERGE [3NF].[Label] AS tgt
USING
(
	SELECT
		Name,
		Headquarters
	FROM
	(
		VALUES
		('Island', 'UK'),
		('E.G.', 'UK')	--Note that if we weren't in 3NF, this data would be lost given our current album list!
	) v(Name, Headquarters)
) AS src
	ON tgt.Name = src.Name
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	Name,
	Headquarters
)
VALUES
(
	src.Name,
	src.Headquarters
);

MERGE [3NF].[Album] AS tgt
USING
(
	SELECT
		Year,
		ProjeKctName,
		Type,
		Label,
		Name
	FROM
	(
		VALUES
		(1998, 'ProjeKct 1', 'Live', NULL, 'Live at the Jazz Cafe'),
		(1997, 'ProjeKct 2', 'Live', NULL, 'Live Groove'),
		(1997, 'ProjeKct 2', 'Studio', NULL, 'Space Groove'),
		(1969, NULL, 'Studio', 'Island', 'In the Court of the Crimson King'),
		(1970, NULL, 'Studio', 'Island', 'In the Wake of Poseidon')
	) v
	(
		Year,
		ProjeKctName,
		Type,
		Label,
		Name
	)
) AS src
	ON tgt.Name = src.Name
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	Year,
	ProjeKctName,
	Type,
	Name,
	LabelName
)
VALUES
(
	src.Year,
	src.ProjeKctName,
	src.Type,
	src.Name,
	src.Label
);

MERGE [3NF].[AlbumXMember] AS tgt
USING
(
	SELECT
		AlbumName,
		MemberName,
		IsSessionOnly
	FROM
	(
		VALUES
		('Live at the Jazz Cafe', 'Tony Levin', 0),
		('Live at the Jazz Cafe', 'Trey Gunn', 0),
		('Live at the Jazz Cafe', 'Bill Bruford', 0),
		('Live at the Jazz Cafe', 'Robert Fripp', 0),
		('Live Groove', 'Adrian Belew', 0),
		('Live Groove', 'Robert Fripp', 0),
		('Live Groove', 'Trey Gunn', 0),
		('Space Groove', 'Adrian Belew', 0),
		('Space Groove', 'Robert Fripp', 0),
		('Space Groove', 'Trey Gunn', 0),
		('In the Court of the Crimson King', 'Robert Fripp', 0),
		('In the Court of the Crimson King', 'Ian McDonald', 0),
		('In the Court of the Crimson King', 'Greg Lake', 0),
		('In the Court of the Crimson King', 'Michael Giles', 0),
		('In the Court of the Crimson King', 'Peter Sinfield', 0),
		('In the Wake of Poseidon', 'Robert Fripp', 0),
		('In the Wake of Poseidon', 'Greg lake', 0),
		('In the Wake of Poseidon', 'Michael Giles', 0),
		('In the Wake of Poseidon', 'Peter Giles', 0),
		('In the Wake of Poseidon', 'Keith Tippett', 1),
		('In the Wake of Poseidon', 'Mel Collins', 0),
		('In the Wake of Poseidon', 'Gordon Haskell', 0),
		('In the Wake of Poseidon', 'Peter Sinfield', 0)
	) v(AlbumName, MemberName, IsSessionOnly)
) AS src
	ON tgt.AlbumName = src.AlbumName
	AND tgt.MemberName = src.MemberName
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	AlbumName,
	MemberName,
	IsSessionOnly
)
VALUES
(
	src.AlbumName,
	src.MemberName,
	src.IsSessionOnly
);

MERGE [3NF].[AlbumXMemberXInstrument] AS tgt
USING
(
	SELECT
		AlbumName,
		MemberName,
		InstrumentName
	FROM
	(
		VALUES
		('Live at the Jazz Cafe', 'Tony Levin', 'Bass'),
		('Live at the Jazz Cafe', 'Tony Levin', 'Stick'),
		('Live at the Jazz Cafe', 'Tony Levin', 'Synth'),
		('Live at the Jazz Cafe', 'Trey Gunn', 'Touch guitar'),
		('Live at the Jazz Cafe', 'Bill Bruford', 'Drums'),
		('Live at the Jazz Cafe', 'Robert Fripp', 'Guitar'),
		('Live Groove', 'Adrian Belew', 'V-Drums'),
		('Live Groove', 'Robert Fripp', 'Guitar'),
		('Live Groove', 'Trey Gunn', 'Touch guitar'),
		('Live Groove', 'Trey Gunn', 'Talker'),
		('Space Groove', 'Adrian Belew', 'V-Drums'),
		('Space Groove', 'Robert Fripp', 'Guitar'),
		('Space Groove', 'Trey Gunn', 'Touch guitar'),
		('Space Groove', 'Trey Gunn', 'Guitar synth'),
		('In the Court of the Crimson King', 'Robert Fripp', 'Guitar'),
		('In the Court of the Crimson King', 'Ian McDonald', 'Reeds'),
		('In the Court of the Crimson King', 'Ian McDonald', 'Woodwind'),
		('In the Court of the Crimson King', 'Ian McDonald', 'Vibes'),
		('In the Court of the Crimson King', 'Ian McDonald', 'Keyboards'),
		('In the Court of the Crimson King', 'Ian McDonald', 'Mellotron'),
		('In the Court of the Crimson King', 'Ian McDonald', 'Vocals'),
		('In the Court of the Crimson King', 'Greg Lake', 'Bass'),
		('In the Court of the Crimson King', 'Greg Lake', 'Vocals'),
		('In the Court of the Crimson King', 'Michael Giles', 'Drums'),
		('In the Court of the Crimson King', 'Michael Giles', 'Percussion'),
		('In the Court of the Crimson King', 'Michael Giles', 'Vocals'),
		('In the Court of the Crimson King', 'Peter Sinfield', 'Words and Illumination'),
		('In the Wake of Poseidon', 'Robert Fripp', 'Guitar'),
		('In the Wake of Poseidon', 'Robert Fripp', 'Mellotron'),
		('In the Wake of Poseidon', 'Robert Fripp', 'Devices'),
		('In the Wake of Poseidon', 'Greg lake', 'Vocals'),
		('In the Wake of Poseidon', 'Michael Giles', 'Drums'),
		('In the Wake of Poseidon', 'Peter Giles', 'Bass'),
		('In the Wake of Poseidon', 'Keith Tippett', 'Piano'),
		('In the Wake of Poseidon', 'Mel Collins', 'Sax'),
		('In the Wake of Poseidon', 'Mel Collins', 'Flute'),
		('In the Wake of Poseidon', 'Gordon Haskell', 'Vocals'),
		('In the Wake of Poseidon', 'Peter Sinfield', 'Words')
	) v(AlbumName, MemberName, InstrumentName)
) AS src
	ON tgt.AlbumName = src.AlbumName
	AND tgt.MemberName = src.MemberName
	AND tgt.InstrumentName = src.InstrumentName
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	AlbumName,
	MemberName,
	InstrumentName
)
VALUES
(
	src.AlbumName,
	src.MemberName,
	src.InstrumentName
);

MERGE [3NF].[Song] AS tgt
USING
(
	SELECT
		[AlbumName],
		[TrackNumber],
		[SongName]
	FROM
	(
		VALUES
		('Live at the Jazz Cafe', 1, '4 i 1'),
		('Live at the Jazz Cafe', 2, '4 ii 1'),
		('Live at the Jazz Cafe', 3, '1 ii 2'),
		('Live at the Jazz Cafe', 4, '4 ii 4'),
		('Live at the Jazz Cafe', 5, '2 ii 3'),
		('Live at the Jazz Cafe', 6, '3 i 2'),
		('Live at the Jazz Cafe', 7, '3 ii 2'),
		('Live at the Jazz Cafe', 8, '2 ii 4'),
		('Live at the Jazz Cafe', 9, '4 i 3'),
		('Live Groove', 1, 'Sus-tayn-Z'),
		('Live Groove', 2, 'Heavy ConstruKction'),
		('Live Groove', 3, 'The Deception of the Thrush'),
		('Live Groove', 4, 'X-chayn-jiZ'),
		('Live Groove', 5, 'Light ConstruKction'),
		('Live Groove', 6, 'Vector Shift to Planet Detroit'),
		('Live Groove', 7, 'Contrary ConstruKction'),
		('Live Groove', 8, 'Live Groove'),
		('Live Groove', 9, 'Vector Shift to Planet Belewbeloid'),
		('Space Groove', 1, 'Space Groove II'),
		('Space Groove', 2, 'Space Groove III'),
		('Space Groove', 3, 'Space Groove I'),
		('Space Groove', 4, 'The Planet Zarg Quartet'),
		('Space Groove', 5, 'Happy Hour on Planet Zarg'),
		('Space Groove', 6, 'Is There Life on Planet Zarg?'),
		('Space Groove', 7, 'Low Life in Sector Q-3'),
		('Space Groove', 8, 'Sector Shift'),
		('Space Groove', 9, 'Laura in Space'),
		('Space Groove', 10, 'Sector Drift'),
		('Space Groove', 11, 'Sector Patrol'),
		('Space Groove', 12, 'In Space There Is No North...'),
		('Space Groove', 13, 'Vector Patrol'),
		('Space Groove', 14, 'Deserts of Arcadia (North)'),
		('Space Groove', 15, 'Deserts of Arcadia (South)'),
		('Space Groove', 16, 'Snake Drummers of Sector Q-e'),
		('Space Groove', 17, 'Escape from Sagittarius A'),
		('Space Groove', 18, 'Return to Station B'),
		('In the Court of the Crimson King', 1, '21st Century Schizoid Man'),
		('In the Court of the Crimson King', 2, 'I Talk to the Wind'),
		('In the Court of the Crimson King', 3, 'Epitaph'),
		('In the Court of the Crimson King', 4, 'moonchild'),
		('In the Court of the Crimson King', 5, 'The Court of the Crimson King'),
		('In the Wake of Poseidon', 1, 'Peace - A Beginning'),
		('In the Wake of Poseidon', 2, 'Pictures of a City'),
		('In the Wake of Poseidon', 3, 'Cadence and Cascade'),
		('In the Wake of Poseidon', 4, 'In the Wake of Poseidon'),
		('In the Wake of Poseidon', 5, 'Peace - A Theme'),
		('In the Wake of Poseidon', 6, 'Cat Food'),
		('In the Wake of Poseidon', 7, 'The Devil''s Triangle'),
		('In the Wake of Poseidon', 8, 'Peace - An End')
	) v
	(
		[AlbumName],
		[TrackNumber],
		[SongName]
	)
) AS src
	ON tgt.[AlbumName] = src.[AlbumName]
	AND tgt.[SongName] = src.[SongName]
WHEN NOT MATCHED BY TARGET THEN INSERT
(
	[AlbumName],
	[TrackNumber],
	[SongName]
)
VALUES
(
	src.AlbumName,
	src.TrackNumber,
	src.SongName
);


--1)  "UK" should now be United Kingdom for Island label's headquarters.
--		In 2NF, we would have needed to update multiple rows; now
--		we only need to update a single row.
UPDATE l
SET
	l.Headquarters = 'United Kingdom'
FROM [3NF].[Label] l
WHERE
	l.Name = 'Island';